{********************************************************}
{*   Active Query Builder Component Suite               *}
{*                                                      *}
{*   Copyright (C) 2005 Active Database Software        *}
{*   All rights reserved.                               *}
{*                                                      *}
{********************************************************}

unit fuAQBuilder;

{$INCLUDE aqbdemo.inc}

interface

uses
	Windows, Classes, Graphics, Forms, Controls, StdCtrls,
	ExtCtrls, ToolWin, ComCtrls, acQBBase, DB, ADODB, SysUtils,
	DBCtrls, Grids, DBGrids, ImgList, Dialogs,
	acSQL92SynProvider, acSQLBuilderPlainText, acQBADOMetaProvider,

	{**********************************************************}
	{* Please note that to compile this demo application with *}
	{* Free Edition of Active Query Builder you should define *}
	{* FREE conditional symbol in 'aqbdemo.inc' file.         *}
	{**********************************************************}

	{$IFNDEF FREE}
	acMYSQLSynProvider, acMSSQLSynProvider, acPGSQLSynProvider,
	acSQLiteSynProvider, acIBSynProvider,
	{$ENDIF}

	{$IFDEF SYNEDIT}
	SynMemo, SynEdit, SynEditHighlighter, SynHighlighterSQL,
	{$ENDIF}

	acAST, DBTables, acQBBDEMetaProvider, Buttons;

type
  TfmAQBuilder = class(TForm)
	 acADOMetadataProvider: TacADOMetadataProvider;
	 acSQLBuilderPlainText: TacSQLBuilderPlainText;
	 ADOConnection: TADOConnection;
	 panelBuilder: TPanel;
	 splVertical: TSplitter;
	 panelBuilderLeft: TPanel;
	 splHorizontal: TSplitter;
	 QueryBuilder: TacQueryBuilder;
	 panelGrid: TPanel;
	 panelGridTop: TPanel;
	 DataSource: TDataSource;
    ADOQuery: TADOQuery;
	 Grid: TDBGrid;
	 DBNavigator: TDBNavigator;
	 panelBuilderTop: TPanel;
	 cbSyntaxProvider: TComboBox;
	 lbSyntaxProvider: TLabel;
	 bvTop: TBevel;
	 Bevel2: TBevel;
	 imgObjects16: TImageList;
	 pnlServerVersion: TPanel;
	 cbMSVersion: TComboBox;
	 edMyVersion: TEdit;
	 treeObjects: TTreeView;
	 imgObjects32: TImageList;
	 pnlBottom: TPanel;
    acBDEMetadataProvider: TacBDEMetadataProvider;
    BDEQuery: TQuery;
    sbShowQueryTree: TSpeedButton;
    BDEDatabase: TDatabase;
	 procedure FormClose(Sender: TObject; var Action: TCloseAction);
	 procedure cbSyntaxProviderChange(Sender: TObject);
	 procedure QueryBuilderDragOver(Sender, Source: TObject;
		X, Y: Integer; State: TDragState; var Accept: Boolean);
	 procedure QueryBuilderDragDrop(Sender, Source: TObject; X, Y: Integer);
	 procedure acSQLBuilderPlainTextSQLUpdated(Sender: TObject);
	 procedure editSQLExit(Sender: TObject);
	 procedure editSQLChange(Sender: TObject);
	 procedure FormActivate(Sender: TObject);
	 procedure FormCreate(Sender: TObject);
	 procedure listTablesDblClick(Sender: TObject);
	 procedure cbMSVersionChange(Sender: TObject);
	 procedure edMyVersionChange(Sender: TObject);
	 procedure QueryBuilderSQLError(Sender: TObject;
		ASQL, AError, AErrToken: String; AErrTokenPos: TacPos);
	 procedure sbShowQueryTreeClick(Sender: TObject);
  private
	 { Private declarations }
	 fSQLChanged: Boolean;
	 fEditorChanged: Boolean;
	 fRun: Boolean;
	 fSyntaxProvider: TacBaseSyntaxProvider;
	 procedure SetRun(const Value: Boolean);
	 procedure SetEditorChanged(const Value: Boolean);
  public
	 { Public declarations }
	 ConnType: Integer;
	 {$IFDEF SYNEDIT}
		 SynSQLSyntaxHighlighter: TSynSQLSyn;
		 editSQL: TSynMemo;
	 {$ELSE}
		 editSQL: TMemo;
	 {$ENDIF}
	 property IsRun: Boolean read fRun write SetRun;
	 property EditorChanged: Boolean read fEditorChanged write SetEditorChanged;
  end;

implementation

{$R *.dfm}

uses fuAQBDemoMain;

procedure TfmAQBuilder.FormCreate(Sender: TObject);
begin
	IsRun := False;
	fSQLChanged := False;
	fEditorChanged := False;

	{$IFDEF SYNEDIT}
		editSQL := TSynMemo.Create(pnlBottom);
		SynSQLSyntaxHighlighter := TSynSQLSyn.Create(Self);
		editSQL.Highlighter := SynSQLSyntaxHighlighter;
	{$ELSE}
		editSQL := TMemo.Create(pnlBottom);
	{$ENDIF}

	with editSQL do begin
		Parent :=  pnlBottom;
		Align := alClient;
		OnExit := editSQLExit;
      OnChange := editSQLChange;
	end;

	with cbSyntaxProvider.Items do begin
		Add('ANSI 92');
		{$IFNDEF FREE}
		Add('Microsoft SQL Server');
		Add('MySQL');
		Add('PostgreSQL');
		Add('Interbase (Firebird)');
		Add('SQLite');
		{$ENDIF}
	end;
	cbSyntaxProvider.ItemIndex := 0;
	cbSyntaxProviderChange(nil);
end;

procedure TfmAQBuilder.FormActivate(Sender: TObject);
begin
	with MainForm do begin
		CurrentQueryBuilder := Self;

		acQueryBuild.Enabled := fRun;
		acQueryRun.Enabled := not fRun;
		acFileSave.Enabled := fEditorChanged;
		acFileSaveAs.Enabled := True;
      sbShowQueryTree.Down := QueryBuilder.TreeOptions.TreeVisible;
	end
end;

procedure TfmAQBuilder.FormClose(Sender: TObject; var Action: TCloseAction);
begin
	Action := caFree;
end;

procedure TfmAQBuilder.cbSyntaxProviderChange(Sender: TObject);
begin
	if Assigned(fSyntaxProvider)
		then FreeAndNil(fSyntaxProvider);

	case cbSyntaxProvider.ItemIndex of
		0: begin
			fSyntaxProvider := TacSQL92SyntaxProvider.Create(Self);
			pnlServerVersion.Visible := True;
		end;

		{$IFNDEF FREE}
		1: begin
			fSyntaxProvider := TacMSSQLSyntaxProvider.Create(Self);
			pnlServerVersion.Visible := True;
			cbMSVersion.BringToFront;
			case TacMSSQLSyntaxProvider(fSyntaxProvider).ServerVersion of
				svMSSQL7: cbSyntaxProvider.ItemIndex := 0;
				svMSSQL2000: cbSyntaxProvider.ItemIndex := 1;
				svMSSQL2005: cbSyntaxProvider.ItemIndex := 2;
			end;
		end;
		2:	begin
			fSyntaxProvider := TacMYSQLSyntaxProvider.Create(Self);
			pnlServerVersion.Visible := True;
			edMyVersion.BringToFront;
			edMyVersion.Text :=
				TacMYSQLSyntaxProvider(fSyntaxProvider).ServerVersion;
		end;
		3: begin
			fSyntaxProvider := TacPGSQLSyntaxProvider.Create(Self);
			pnlServerVersion.Visible := False;
		end;
		4: begin
			fSyntaxProvider := TacIBSyntaxProvider.Create(Self);
			pnlServerVersion.Visible := False;
		end;
		5: begin
			fSyntaxProvider := TacSQLiteSyntaxProvider.Create(Self);
			pnlServerVersion.Visible := False;
		end;
		{$ENDIF}
	end;

	QueryBuilder.SyntaxProvider := fSyntaxProvider;
end;

procedure TfmAQBuilder.QueryBuilderDragOver(Sender, Source: TObject;
  X, Y: Integer; State: TDragState; var Accept: Boolean);
begin
	Accept := (Source = treeObjects) and
				 (treeObjects.Selected.ImageIndex < 2);
end;

procedure TfmAQBuilder.QueryBuilderDragDrop(Sender, Source: TObject;
  X, Y: Integer);
var
	ds: TacDataSource;
	usq: TacUnionSubQuery;
	objName: string;
begin
	usq := Sender as TacUnionSubQuery;
	usq.BeginUpdate;
	try
		objName := treeObjects.Selected.Text;
		ds := usq.AddObjectAt(objName, Point(x, y));
		try
			if ds is TacDataSourceObject
				then usq.AddFKLinksForObject(TacDataSourceObject(ds))
		except ;
		end;
	finally
		usq.EndUpdate;
	end;
end;

procedure TfmAQBuilder.listTablesDblClick(Sender: TObject);
var
	ds: TacDataSource;
	usq: TacUnionSubQuery;
	objName: string;
begin
	if treeObjects.Selected.ImageIndex = 2 then Exit;

	usq := QueryBuilder.ActiveSubQuery.ActiveUnionSubquery;
	usq.BeginUpdate;
	try
		objName := treeObjects.Selected.Text;
		ds := usq.AddObject(objName);
		try
			if ds is TacDataSourceObject
				then usq.AddFKLinksForObject(TacDataSourceObject(ds));
		except ;
		end;
	finally
		usq.EndUpdate;
	end;
end;

procedure TfmAQBuilder.acSQLBuilderPlainTextSQLUpdated(Sender: TObject);
begin
	if not Assigned(editSQL) then Exit;
	editSQL.Lines.Text := acSQLBuilderPlainText.SQL;
	EditorChanged := True;
end;

procedure TfmAQBuilder.editSQLExit(Sender: TObject);
begin
	if fSQLChanged then begin
		QueryBuilder.SQL := editSQL.Text;
		fSQLChanged := False;
	end;
end;

procedure TfmAQBuilder.editSQLChange(Sender: TObject);
begin
	fSQLChanged := True;
	EditorChanged := True;
end;

procedure TfmAQBuilder.SetRun(const Value: Boolean);
begin
	fRun := Value;
	if Value
	then case ConnType of
		0: begin
			DataSource.DataSet := ADOQuery;
			ADOQuery.SQL.Assign(editSQL.Lines);
			ADOQuery.Prepared := True;
			if ADOQuery.Prepared
			then begin
				ADOQuery.Open;
				panelGrid.BringToFront;
			end;
		end;
		1: begin
			DataSource.DataSet := BDEQuery;
			BDEQuery.SQL.Assign(editSQL.Lines);
			BDEQuery.Prepared := True;
			if BDEQuery.Prepared
			then begin
				BDEQuery.Open;
				panelGrid.BringToFront;
			end;
		end;
	end
	else begin
		if ADOQuery.Active then ADOQuery.Close;
		panelBuilder.BringToFront;
	end;
	FormActivate(nil);
end;

procedure TfmAQBuilder.SetEditorChanged(const Value: Boolean);
begin
  fEditorChanged := Value;
  FormActivate(nil);
end;

procedure TfmAQBuilder.cbMSVersionChange(Sender: TObject);
begin
	{$IFNDEF FREE}
	if fSyntaxProvider is TacMSSQLSyntaxProvider then
	with fSyntaxProvider as TacMSSQLSyntaxProvider do begin

		case cbMSVersion.ItemIndex of
			0: ServerVersion := svMSSQL7;
			1: ServerVersion := svMSSQL2000;
			2: ServerVersion := svMSSQL2005;
		end;
	end;
  {$ENDIF}
end;

procedure TfmAQBuilder.edMyVersionChange(Sender: TObject);
begin
  {$IFNDEF FREE}
	if fSyntaxProvider is TacMySQLSyntaxProvider then
	with fSyntaxProvider as TacMySQLSyntaxProvider do begin

		ServerVersion := edMyVersion.Text;
		edMyVersion.Text := ServerVersion;
	end
	{$ENDIF}
end;

procedure TfmAQBuilder.QueryBuilderSQLError(Sender: TObject;
  ASQL, AError, AErrToken: String; AErrTokenPos: TacPos);
begin
	ShowMessage(AError);
	editSQL.SelStart := AErrTokenPos.pos + 1;
	editSQL.SetFocus;
end;

procedure TfmAQBuilder.sbShowQueryTreeClick(Sender: TObject);
begin
	QueryBuilder.TreeOptions.TreeVisible := sbShowQueryTree.Down;
end;

end.
